<?php

namespace Drupal\packagist_repos\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\packagist_repos\Entity\RepositoryInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for repository package actions ajax requests.
 */
class RepositoryAjaxActionsController extends ControllerBase {

  /**
   * Return response with action execution status info.
   *
   * @return JsonResponse
   *   Return status_info response.
   */
  public function response(RepositoryInterface $packagist_repository, $action) {
    // @todo: check permissions for the user and given entity
    // @todo: check if the entity belongs to the current user
    //   add check to the access check callback

    // @todo: make changes to the resultant structure if needed,
    //   explicitly list response data keys since it actually
    //   describes the api conventions used on the client-side,
    //   also some validation could be appropriate here

    // @todo: message should be set here based on status code
    //   also the status code to be replaced here by text as well
    switch ($action) {
      case 'status':
        $status_info = $packagist_repository->runGetStatus();
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          'data' => $status_info['data'],
        ];
        break;
      // @todo: add git_fetch action to update repository
      //   also check for non-fast-forward changes, removed/new tags and branches,
      //   other significant changes and data
      case 'git_clone':
        $status_info = $packagist_repository->runGitClone();
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          'data' => $status_info['data'],
        ];
        break;
      case 'git_fetch':
        $status_info = $packagist_repository->runGitFetch();
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          //'data' => $status_info['data'],
        ];
        break;
      case 'git_snapshot_update':
        $status_info = $packagist_repository->runGitSnapshotUpdate();
        // @todo: though these status and message are for client-side which are
        //   actually different (by its sense from status and message used internally
        //   since the action is about "to update snapshot" but to about "to get data"
        //   which is done using "status" action)
        //   and should be defined by js api definition
        unset($status_info['data']);
        // just return status and message
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          //'data' => $status_info['data'],
        ];
        break;
      case 'import_tags':
        $status_info = $packagist_repository->runImportTags();
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          'data' => $status_info['data'],
        ];
        break;
      // @todo: maybe rename to publish (though can cause confusion with
      //   generating packagist json structure)
      case 'include_tags':
        $status_info = $packagist_repository->runIncludeTags();
        // just return status and message
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          //'data' => $status_info['data'],
        ];
        break;
      case 'generate_archives':
        $status_info = $packagist_repository->runGenerateArchives();
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          'data' => $status_info['data'],
        ];
        break;
      // @todo: rename to 'update_definition' for 'update package definition'
      case 'generate_packagist_json_structure':
        $status_info = $packagist_repository->runGeneratePackagistJsonStructure();
        $response = [
          'status' => $status_info['status'],
          'message' => $status_info['message'],
          'data' => $status_info['data'],
        ];
        break;
      default:
        // @todo: emit an error
        //   or return 404 response
        $response = [
          'status' => '0',
          'message' => t('Action not supported'),
        ];
    }

    return new JsonResponse($response);
  }

}
